package com.chaosserver.bilbo.data;

import java.io.IOException;
import java.io.Reader;
import java.io.UnsupportedEncodingException;
import java.io.Writer;
import java.util.Iterator;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

import org.apache.crimson.tree.ElementNode;
import org.apache.crimson.tree.XmlWriteContext;

import org.w3c.dom.Attr;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.Text;

import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

/**
 * Responsible for persistence of the Directory structure to/from an Xml
 * mapping file.
 *
 * @author Jordan Reed
 * @version 1.0
 * @since Bilbo 1.0
 */
public class DataReaderWriter {
	/** Xml tag for the Directory object. */
	public static final String DIRECTORY_TAG = "directory";

	/** Xml tag for the FileMapping object. */
	public static final String FILE_TAG = "file";

	/** Xml tag for the Name object. */
	public static final String NAME_TAG = "name";

	/** Xml attribute for the style property of the name attribute. */
	public static final String STYLE_ATT = "style";

	/**
	 * Holds teh document being constructed for writing.  This is needed
	 * for the creation of nodes on the document.
	 */
	protected Document document;

	/**
	 * Default constructor.
	 */
	public DataReaderWriter() {
	}

	/**
	 * Writes the Directory object as Xml to the given writier.
	 *
	 * @param writer the writer to put the xml output into
	 * @param directory the directory to convert to xml and write
	 * @throws ReaderWriterException if there is an error
	 */
	public void write(Writer writer, Directory directory) throws ReaderWriterException {
		try {
			DocumentBuilderFactory documentBuilderFactory = DocumentBuilderFactory.newInstance();
			DocumentBuilder documentBuilder = documentBuilderFactory.newDocumentBuilder();
			Document document = documentBuilder.newDocument();
			this.document = document;

			Element rootElement = document.createElement(DIRECTORY_TAG);
			document.appendChild(rootElement);


			Iterator filenameMappingIterator = directory.getFilenameMappings().iterator();
			while(filenameMappingIterator.hasNext()) {
				FilenameMapping filenameMapping = (FilenameMapping) filenameMappingIterator.next();
				addFilenameMapping(rootElement, filenameMapping);
			}

			writer.write("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n");

			XmlWriteContext context = new XmlWriteContext(writer, 0);
			ElementNode root = (ElementNode) document.getDocumentElement();
			root.writeXml(context);

			this.document = null;

			writer.flush();
		}
		catch (ParserConfigurationException e) {
			this.document = null;
			throw new ReaderWriterException("Failed to build parser", e);
		}
		catch (IOException e) {
			this.document = null;
			throw new ReaderWriterException("Failed to write", e);
		}
	}


	/**
	 * Generates a new node for the FilenameMapping and appends it the node passed in.
	 *
	 * @param node the node to append to
	 * @param filenameMapping the filenameMapping to convert to a node
	 *
	 * @throws ReaderWriterException if something goes wrong
	 */
	protected void addFilenameMapping(Node node, FilenameMapping filenameMapping) throws ReaderWriterException {
		Element filenameMappingElement = document.createElement(FILE_TAG);

		Iterator namesIterator = filenameMapping.getNames().iterator();
		while(namesIterator.hasNext()) {
			Name name = (Name) namesIterator.next();
			addName(filenameMappingElement, name);
		}

		node.appendChild(filenameMappingElement);
	}

	/**
	 * Generates a new node the Name and appends it to the node passed in.
	 *
	 * @param node the node to append to
	 * @param name the name to convert to a node
	 */
	protected void addName(Node node, Name name) throws ReaderWriterException {
		Element nameElement = document.createElement(NAME_TAG);

		Attr styleAttribute = document.createAttribute(STYLE_ATT);
		styleAttribute.setValue(name.getStyle());
		NamedNodeMap attributes = nameElement.getAttributes();
		attributes.setNamedItem(styleAttribute);

		try {
			Text nameValue = document.createTextNode( new String(name.getValue().getBytes("UTF-8")) );
			nameElement.appendChild(nameValue);
		}
		catch(UnsupportedEncodingException e) {
			throw new ReaderWriterException("Failed to encode", e);
		}

		node.appendChild(nameElement);
	}

	/**
	 * Reads in a Xml version of a directory and demarshalls it into the object.
	 *
	 * @param reader the reader that the Xml comes from
	 * @return the Directory represented by the Xml
	 *
	 * @throws ReaderWriterException if something goes wrong.
	 */
	public Directory read(Reader reader) throws ReaderWriterException {
		try {
			DirectoryHandler directoryHandler = new DirectoryHandler();

			InputSource inputSource = new InputSource(reader);

			SAXParserFactory saxParserFactory = SAXParserFactory.newInstance();
			SAXParser saxParser = saxParserFactory.newSAXParser();
			saxParser.parse(inputSource, directoryHandler);

			return directoryHandler.getDirectory();
		}
		catch (ParserConfigurationException e) {
			throw new ReaderWriterException("Failed to configure parser", e);
		}
		catch (SAXException e) {
			throw new ReaderWriterException("Failed to parse", e);
		}
		catch (IOException e) {
			throw new ReaderWriterException("Failed to read", e);
		}
	}

}