package com.chaosserver.bilbo.data;

import java.io.File;
import java.io.FileReader;
import java.util.Collection;
import java.util.ArrayList;
import java.util.Iterator;
import java.io.FileNotFoundException;

/**
 * Directory object represents the mappings contained in a single directory.
 * <p>
 * Holds a collection of FilenameMapping objects that represent the files
 * in the directory that need to be mapped.
 * </p>
 *
 * @author Jordan Reed
 * @version 1.1
 * @since Bilbo 1.0
 */
public class Directory {
	/** Holds the collection of FilenameMappings in this directory. */
	protected Collection filenameMappings;

	/**
	 * Default constructor.
	 */
	public Directory() {
		filenameMappings = new ArrayList();
	}

	/**
	 * Unmarshall a directory object from a mapping file.
	 *
	 * @param mappingFile mapping file to unmarshall
	 * @return directory object held in the file
	 * @throws NoMappingFileException if the mapping file does not exist
	 * @throws ReaderWriterException if an error occurs attempting to unmarshall the file
	 */
	public static Directory getDirectory(File mappingFile) throws NoMappingFileException, ReaderWriterException {
		try {
			FileReader reader = new FileReader(mappingFile);
			DataReaderWriter dataReaderWriter = new DataReaderWriter();
			Directory directory = dataReaderWriter.read(reader);
			return directory;
		}
		catch (FileNotFoundException e) {
			throw new NoMappingFileException("Failed to find file: " + mappingFile);
		}
	}

	/**
	 * Setter method for the filename mappings.
	 *
	 * @param filenameMappings the value of filenameMappings
	 */
	public void setFilenameMappings(Collection filenameMappings) {
		this.filenameMappings = filenameMappings;
	}

	/**
	 * Getter method for the filename mappings.
	 *
	 * @return the value of filenameMappings
	 */
	public Collection getFilenameMappings() {
		return this.filenameMappings;
	}

	/**
	 * Adds a new FilenameMapping object to this directory.
	 *
	 * @param filenameMapping the new FilenameMapping to add
	 */
	public void addFilenameMapping(FilenameMapping filenameMapping) {
		filenameMappings.add(filenameMapping);
	}

	/**
	 * Finds a FilenameMapping held in this Directory.
	 *
	 * @param style the style of the filename being searched for
	 * @param value the value of the filename being searched for
	 *
	 * @throws FilenameMappingNotFoundException if the mapping cannot be found
	 */
	public FilenameMapping findFilenameMapping(String style, String value) throws FilenameMappingNotFoundException {
		Iterator filenameMappingsIterator = getFilenameMappings().iterator();
		FilenameMapping filenameMapping = null;

		FILENAME_ITER: while(filenameMappingsIterator.hasNext()) {
			filenameMapping = (FilenameMapping) filenameMappingsIterator.next();

			try {
				Name name = filenameMapping.findName(style, value);
				break FILENAME_ITER;
			}
			catch (NameNotFoundException e) {

			}

			filenameMapping = null;
		}

		if(filenameMapping == null) {
			throw new FilenameMappingNotFoundException("Error locating filenameMapping.  Search criteria: "
				+ "style=" + style + ", value=" + value);
		}

		return filenameMapping;
	}
}