package com.chaosserver.bilbo.task;

import com.chaosserver.bilbo.help.HelpException;
import com.chaosserver.bilbo.help.HelpResourceNotFoundException;
import com.chaosserver.bilbo.help.Helper;

import java.io.File;
import java.io.IOException;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * Base task that provides the template design pattern for
 * concrete implementations.
 *
 * @author Jordan Reed
 * @version 1.0
 * @since Bilbo 2.0
 */
public abstract class BaseTask implements Task {
	/** Holds the resource location of the default task help file. */
	protected static String DEFAULT_HELP = "com/chaosserver/bilbo/task/nohelp.txt";

	/** Holds the settings object. */
	protected Settings settings;

	/**
	 * Setter for the settings object.
	 *
	 * @param settings the settings object
	 */
	public void setSettings(Settings settings)  {
		this.settings = settings;
	}

	/**
	 * Getter for the settings object.
	 *
	 * @return the settings object
	 */
	protected Settings getSettings() {
		return this.settings;
	}

	/**
	 * Prints the usage notes out for the concrete task.
	 * <p>
	 * The concrete task should place it's usage notes in a file
	 * named <code>help.txt</code> inside its package.  If
	 * no file is found, then the default usage notes will be
	 * displayed.
	 * </p>
	 *
	 * @throws TaskException if there is an error printing
	 *         the usage notes
	 */
	public void printHelp() throws TaskException {
		try {
			try {
				Helper.printHelp(System.out, getHelpResource());
			}
			catch (HelpResourceNotFoundException e) {
				Helper.printHelp(System.out, DEFAULT_HELP);
			}
		}
		catch(HelpException e1) {
			throw new TaskException("Failed to print help.", e1);
		}

	}

	/**
	 * Executes this task against the rootDirectory provided in
	 * the settings object.
	 * <p>
	 * This task will at least be executed against the rootDirectory
	 * in the settings object.  If the recursive flag is on, then it
	 * will also be executed against each subdirectory.
	 * </p>
	 *
	 * @throws TaskException if something goes wrong
	 */
	public void execute() throws TaskException {
		Settings settings = this.getSettings();

		if(settings.isHelp()) {
			printHelp();
		}
		else if(settings.isRecursive()) {
			this.executeRecursive(settings.getRootDirectory());
		}
		else {
			String directoryName = settings.getRootDirectory().getAbsolutePath();
			try {
				directoryName = settings.getRootDirectory().getCanonicalPath();
			}
			catch (IOException e) {
				// Eat it.  Yum!
			}
			System.out.println("Executing on: " + directoryName);
			this.execute(settings.getRootDirectory());
		}
	}

	/**
	 * Executes this task against the given directory, then recursive
	 * against all subdirectories.
	 *
	 * @param currentDirectory the directory to execute the task against
	 * @throws TaskException is something goes wrong
	 */
	protected void executeRecursive(File currentDirectory) throws TaskException {
		String directoryName = currentDirectory.getAbsolutePath();
		try {
			directoryName = currentDirectory.getCanonicalPath();
		}
		catch (IOException e) {
			// Eat it.  Yum!
		}
		System.out.println("Executing on: " + directoryName);
		this.execute(currentDirectory);

		File[] files = currentDirectory.listFiles();
		for(int i=0; i<files.length; i++) {
			if(files[i].isDirectory()) {
				executeRecursive(files[i]);
			}
		}
	}

	/**
	 * Executes this task against the given directory
	 *
	 * @param currentDirectory the directory to execute the task against
	 * @throws TaskException is something goes wrong
	 */
	protected abstract void execute(File currentDirectory) throws TaskException;

	/**
	 * Gets the help resource location.
	 * <p>
	 * This appends "help.txt" to the package location.
	 * </p>
	 *
	 * @return help resource location
	 */
	protected String getHelpResource() {
		String packageName = getClass().getPackage().getName();
		Pattern p = Pattern.compile("\\.");
		Matcher m = p.matcher(packageName);

		String resourceLocation = m.replaceAll("/") + "/help.txt";

		return resourceLocation;
	}
}