package com.chaosserver.bilbo.task;

import java.io.File;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.logging.Logger;

/**
 * Holds the settings for the current execution of Bilbo.
 *
 * @author Jordan Reed
 * @version 1.1
 * @since Bilbo 1.0
 */
public class Settings {
	/** Holds the logger. */
	protected static Logger logger = Logger.getLogger(Settings.class.getName());

	/** Holds the root directory a task is being executing on. */
	protected File rootDirectory = new File(".");

	/** Holds the verbosity flag. */
	protected boolean verbose = false;

	/** Holds the name of the task being executed. */
	protected String taskName = "there";

	/** Holds the argument string being passed to the task. */
	protected String taskArgs = "original";

	/** Holds if the recursive flag. */
	protected boolean recursive = true;

	/** Holds the name of the mapping file. */
	protected String fileMappingName = ".filename-mapping.xml";

	/** Holds the help flag. */
	protected boolean help = false;

	/**
	 * Default constructor.
	 */
	public Settings() {}

	/**
	 * Setter for the name of the task.
	 *
	 * @param taskName the name of the task.
	 */
    protected void setTaskName(String taskName) {
		this.taskName = taskName;
	}

	/**
	 * Getter for the name of the task.
	 *
	 * @return the name of the task.
	 */
	public String getTaskName() {
		return this.taskName;
	}

	/**
	 * Setter for the verbosity flag.
	 *
	 * @param verbose if verbosity is on
	 */
	protected void setVerbose(boolean verbose) {
		this.verbose = verbose;
	}

	/**
	 * Getter for the verbosity flag.
	 *
	 * @return if verbosity is on
	 */
	public boolean isVerbose() {
		return this.verbose;
	}

	/**
	 * Setter for the recursive flag.
	 *
	 * @param recursive value of the recursive flag
	 */
	protected void setRecursive(boolean recursive) {
		this.recursive = recursive;
	}

	/**
	 * Getter for the recursive flag.
	 *
	 * @return value of the recursive flag
	 */
	public boolean isRecursive() {
		return this.recursive;
	}

	/**
	 * Setter for the task argument string.
	 *
	 * @param taskArgs the task argument string
	 */
	protected void setTaskArgs(String taskArgs) {
		this.taskArgs = taskArgs;
	}

	/**
	 * Getter for the task argument string.
	 *
	 * @return the task argument string
	 */
	public String getTaskArgs() {
		return this.taskArgs;
	}

	/**
	 * Setter for the root directory.
	 *
	 * @param rootDirectory the root directory
	 */
	public void setRootDirectory(File rootDirectory) {
		this.rootDirectory = rootDirectory;
	}

	/**
	 * Getter for the root directory.
	 *
	 * @return the root directory
	 */
	public File getRootDirectory() {
		return rootDirectory;
	}

	/**
	 * Setter for the file mapping name.
	 *
	 * @param rootDirectory the file mapping name
	 */
	public void setFileMappingName(String fileMappingName) {
		this.fileMappingName = fileMappingName;
	}

	/**
	 * Getter for the file mapping name.
	 *
	 * @return the file mapping name
	 */
	public String getFileMappingName() {
		return this.fileMappingName;
	}

	/**
	 * Setter for the help property
	 *
	 * @param help the help property
	 */
	protected void setHelp(boolean help) {
		this.help = help;
	}

	/**
	 * Getter for the help property
	 *
	 * @return the help property
	 */
	public boolean isHelp() {
		return this.help;
	}

	/**
	 * Parses the argument string and sets the various properties.
	 *
	 * @param args the array of command line arguments.
	 */
	public void parseArguments(String args[]) throws BadCommandArgumentsException {
		String argString = generateArgString(args);

		// System.out.println("argString = " + argString);

		// Split into flags, command, command-flags
		Pattern p = Pattern.compile("^((?:\\s*\\-\\S+\\s)*)(\\s*[^\\-\\s]+\\s*)(.*)$");
		Matcher m = p.matcher(argString);
		boolean found = m.matches();

		if(!found) {
			throw new BadCommandArgumentsException("Failed to parse: " + argString);
		}

		String mainArgs = m.group(1).trim();
		String taskName = m.group(2).trim();
		String taskArgs = m.group(3).trim();

		// System.out.println("mainArgs = " + mainArgs);
		// System.out.println("taskName = " + taskName);
		// System.out.println("taskArgs = " + taskArgs);

		parseMainArguments(mainArgs);


		setTaskName(taskName);
		setTaskArgs(taskArgs);
	}

	/**
	 * Parses the main arguments in the argument string.
	 *
	 * @param mainArgs Arguments appearing before the task name.
	 */
	protected void parseMainArguments(String mainArgs) {
		if(Pattern.matches("\\-\\S*v.*", mainArgs)) {
			setVerbose(true);
		}

		if(Pattern.matches("\\-\\S*c.*", mainArgs)) {
			setRecursive(false);
		}

		if(Pattern.matches("\\-\\S*h.*", mainArgs)) {
			setHelp(true);
		}

	}

	/**
	 * Converts the arguments array into a single string.
	 *
	 * @param args The arguments array
	 * @return A string of the array.
	 */
	protected String generateArgString(String args[]) {
		StringBuffer argumentString = new StringBuffer();

		for(int i=0; i<args.length; i++) {
			argumentString.append(args[i]);

			if(i+1 < args.length) {
				argumentString.append(" ");
			}
		}

		return argumentString.toString();

	}
}