package com.chaosserver.bilbo.task;

import java.beans.XMLDecoder;
import java.io.InputStream;
import java.util.HashMap;
import java.util.Map;

/**
 * Factory class to produce instance of Task objects for task names.
 *
 * @author Jordan Reed
 * @version 1.0
 * @since Bilbo 2.0
 */
public class TaskFactory {
	/** Self reference for singleton. */
	public static TaskFactory self;

	/** Sync object for instance. */
	public static String SYNC = "SYNC";

	/** Holds the map of task name to task class names. */
	protected Map taskMap;

	/**
	 * Default constructor.
	 */
	protected TaskFactory() {
		assert (self==null) : "Singleton cannot be made twice";

		this.loadTaskMap();
	}

	/**
	 * Gets the singleton instance of the TaskFactory.
	 *
	 * @return the singleton instance of the TaskFactory
	 */
	public static TaskFactory getInstance() {
		if(self==null) {
			synchronized(SYNC) {
				if(self==null) {
					self = new TaskFactory();
				}
			}
		}

		return self;
	}

	/**
	 * Getter for the taskMap property.
	 *
	 * @return the taskMap property
	 */
	protected Map getTaskMap() {
		return this.taskMap;
	}

	/**
	 * Setter for the taskMap property.
	 *
	 * @param taskMap the taskMap property
	 */
	protected void setTaskMap(Map taskMap) {
		this.taskMap = taskMap;
	}

	/**
	 * Loads the taskMap property from file.
	 */
	protected void loadTaskMap() {
		ClassLoader classLoader = this.getClass().getClassLoader();
		InputStream inputStream = classLoader.getResourceAsStream("com/chaosserver/bilbo/task/TaskFactoryMap.xml");
		XMLDecoder xmlDecoder = new XMLDecoder(inputStream);
        Map taskMap = (Map) xmlDecoder.readObject();
        xmlDecoder.close();

        setTaskMap(taskMap);
	}


	/**
	 * Creates a new instance of the task object associated with the taskName given.
	 *
	 * @param taskName the name of the task object to create
	 * @param settings the settings of this instance of the program
	 * @throws NoSuchTaskException if there is not task for the task name given
	 * @throws TaskCreationException if there is an error instantiating the task
	 */
	public Task createTask(String taskName, Settings settings) throws NoSuchTaskException, TaskCreationException {
		Map taskMap = this.getTaskMap();
		if(!taskMap.containsKey(taskName)) {
			throw new NoSuchTaskException("Failed to find a task by name: " + taskName);
		}

		String taskClassName = (String) getTaskMap().get(taskName);
		try {
			Class taskClass = Class.forName(taskClassName);
			Task task = (Task) taskClass.newInstance();
			task.setSettings(settings);
			return task;
		}
		catch (ClassNotFoundException e) {
			throw new TaskCreationException("Failed to create task: " + taskClassName, e);
		}
		catch (InstantiationException e) {
			throw new TaskCreationException("Failed to instantiate task: " + taskClassName, e);
		}
		catch (IllegalAccessException e) {
			throw new TaskCreationException("Bad access writes to task: " + taskClassName, e);
		}

	}
}

