package com.chaosserver.bilbo.task.convert;

import com.chaosserver.bilbo.data.DataReaderWriter;
import com.chaosserver.bilbo.data.Directory;
import com.chaosserver.bilbo.data.FilenameMapping;
import com.chaosserver.bilbo.data.FilenameMappingNotFoundException;
import com.chaosserver.bilbo.data.Name;
import com.chaosserver.bilbo.data.NameNotFoundException;
import com.chaosserver.bilbo.data.ReaderWriterException;
import com.chaosserver.bilbo.task.BadTaskCommandArgumentsException;
import com.chaosserver.bilbo.task.BaseTask;

import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * Tasked used to convert between two different mappings.
 *
 * @author Jordan Reed
 * @version 1.0
 * @since Bilbo 2.0
 */
public class ConvertTask extends BaseTask {
	/** Holds the mapping style files start in. */
	protected String fromStyle;

	/** Holds the mapping style files end in. */
	protected String toStyle;

	/**
	 * Holds the error level.
	 * <ul>
	 *   <li>0 - warn on errors</li>
	 *   <li>1 - prompt on errors</li>
	 *   <li>2 - throw exception on errors</li>
	 * </ul>
	 */
	protected int errorLevel = 0;

	/**
	 * Setter for the fromStyle property.
	 *
	 * @param fromStyle the fromStyle property
	 */
	protected void setFromStyle(String fromStyle) {
		this.fromStyle = fromStyle;
	}

	/**
	 * Getter for the fromStyle property.
	 *
	 * @return the fromStyle property
	 */
	protected String getFromStyle() {
		return this.fromStyle;
	}

	/**
	 * Setter for the toStyle property.
	 *
	 * @param toStyle the toStyle property
	 */
	protected void setToStyle(String toStyle) {
		this.toStyle = toStyle;
	}

	/**
	 * Getter for the toStyle property.
	 *
	 * @return the toStyle property
	 */
	protected String getToStyle() {
		return this.toStyle;
	}

	/**
	 * Setter for the errorLevel property.
	 *
	 * @param errorLevel the errorLevel property
	 */
	protected void setErrorLevel(int errorLevel) {
		this.errorLevel = errorLevel;
	}

	/**
	 * Getter for the errorLevel property.
	 *
	 * @return the errorLevel property
	 */
	protected int getErrorLevel() {
		return this.errorLevel;
	}

	// inherits javadocs
	public void parseTaskArguments(String argumentString) throws BadTaskCommandArgumentsException {
		// Split into flags, from style, to style
		Pattern p = Pattern.compile("^((?:\\s*\\-\\S+\\s)*)\\s*(\\S+)\\s+(\\S+)\\s*$");
		Matcher m = p.matcher(argumentString);
		boolean found = m.matches();

		if(!found) {
			throw new BadTaskCommandArgumentsException("Failed to parse argument string: " + argumentString);
		}

		String flags = m.group(1).trim();
		String fromStyle = m.group(2).trim();
		String toStyle = m.group(3).trim();

		// System.out.println("flags: " + flags);
		// System.out.println("fromStyle: " + fromStyle);
		// System.out.println("toStyle: " + toStyle);

		parseFlags(flags);
		setFromStyle(fromStyle);
		setToStyle(toStyle);
	}

	/**
	 * Parses the task specific flags.
	 *
	 * @param flags the task specific flags
	 */
	protected void parseFlags(String flags) {
		Pattern p;
		Matcher m;

		p = Pattern.compile("\\-\\S*e(\\d).*");
		m = p.matcher(flags);
		if(m.matches()) {
			int errorLevel = Integer.parseInt(m.group(1));
			setErrorLevel(errorLevel);
		}
	}

	/**
	 * Swaps the files in the directory.
	 * <ol>
	 *   <li>File the file name in the directory by fromStyle</li>
	 *   <li>Rename the file in the directory to the toStyle</li>
	 * </ol>
	 *
	 * @param currentDirectory the directory to execute against
	 * @throws ConvertException if something goes wrong
	 */
	public void execute(File currentDirectory) throws ConvertException {
		File mappingFile = new File(currentDirectory, getSettings().getFileMappingName());

		try {
			// Unmarshal it in
			Directory directory = Directory.getDirectory(mappingFile);

			// Now let's rename!
			File[] files = currentDirectory.listFiles();

			FILE_ITER: for(int i=0; i<files.length; i++) {
				String fileName = files[i].getName();
				if(getSettings().getFileMappingName().equals(fileName)) {
					continue FILE_ITER;
				}

				try {
					FilenameMapping filenameMapping = directory.findFilenameMapping(getFromStyle(), fileName);
					Name newName = filenameMapping.findName(getToStyle());

					if(getSettings().isVerbose()) {
						System.out.println("Rename: ");
						System.out.println("  From: " + files[i].getName());
						System.out.println("  To  : " + newName.getValue());
						System.out.println("");
					}

					if(!fileName.equals(newName.getValue())) {
						files[i].renameTo(new File(files[i].getParentFile(), newName.getValue()));
					}
				}
				catch (FilenameMappingNotFoundException e) {
					String errorMessage = "Failed to find mapping for: " + fileName;

					if(getErrorLevel() == 0) {
						System.err.println("warning: " + errorMessage);
					}
					else {
						throw new ConvertException(errorMessage, e);
					}
				}
				catch (NameNotFoundException e) {
					String errorMessage = "Couldn't find destination style for: " + fileName;
					if(getErrorLevel() == 0) {
						System.err.println("warning: " + errorMessage);
					}
					else {
					  throw new ConvertException(errorMessage, e);
				  	}
				}
			}
		}
		catch (com.chaosserver.bilbo.data.NoMappingFileException e) {
			String filename = mappingFile.getAbsolutePath();
			try {
				filename = mappingFile.getCanonicalPath();
			}
			catch(IOException e1) {
				// Eat it.  Yum!
			}
			String errorMessage = "could not find mapping file: " + filename;

			if(getErrorLevel() == 0) {
				System.err.println("warning: " + errorMessage);
			}
			else {
				throw new ConvertException(errorMessage, e);
			}
		}
		catch (ReaderWriterException e) {
			throw new ConvertException("Error unmarshalling the file.", e);
		}
	}
}