package com.chaosserver.bilbo.task.generate;

import java.util.Map;
import java.beans.XMLDecoder;
import java.io.InputStream;

/**
 * Factory object used to create formatters.
 *
 * @author Jordan Reed
 * @version 1.0
 * @since Bilbo 2.0
 */
public class FormatterFactory {
	/** Self reference for singleton. */
	protected static FormatterFactory self;

	/** Synchronization object. */
	protected static String SYNC = "SYNC";

	/** Holds the map of formatter names to formatter class names. */
	protected Map formatterMap;

	/**
	 * Default constructor.
	 */
	protected FormatterFactory() {
		assert(self==null) : "Only one singleton can be created.";
		this.loadFormatterMap();
	}

	/**
	 * Setter for the formatterMap
	 *
	 * @param formatterMap the formatterMap
	 */
	protected void setFormatterMap(Map formatterMap) {
		this.formatterMap = formatterMap;
	}

	/**
	 * Getter for the formatterMap
	 *
	 * @return the formatterMap
	 */
	protected Map getFormatterMap() {
		return this.formatterMap;
	}

	/**
	 * Loads the formatter map from file.
	 */
	protected void loadFormatterMap() {
		ClassLoader classLoader = this.getClass().getClassLoader();
		InputStream inputStream = classLoader.getResourceAsStream("com/chaosserver/bilbo/task/generate/FormatterFactoryMap.xml");
		XMLDecoder xmlDecoder = new XMLDecoder(inputStream);
        Map formatterMap = (Map) xmlDecoder.readObject();
        xmlDecoder.close();

        setFormatterMap(formatterMap);
	}

	/**
	 * Gets the instance of the FormatterFactory.
	 *
	 * @return the FormatterFactory
	 */
	public static FormatterFactory getInstance() {
		if(self==null) {
			synchronized(SYNC) {
				if(self==null) {
					self = new FormatterFactory();
				}
			}
		}

		return self;
	}

	/**
	 * Gets an instance of a formatter object for the name given.
	 *
	 * @param formatterName the name of the formatter to get
	 * @return a formatter for the name given
	 * @throws NoSuchFormatterException if there is no formatter for this name
	 * @throws FormatterCreationException if there is an error instantiating the formatter
	 */
	public Formatter getFormatter(String formatterName) throws NoSuchFormatterException, FormatterCreationException {
		Map formatterMap = this.getFormatterMap();
		if(!formatterMap.containsKey(formatterName)) {
			throw new NoSuchFormatterException("Failed to find a formatter by name: " + formatterName);
		}

		String formatterClassName = (String) formatterMap.get(formatterName);
		try {
			Class formatterClass = Class.forName(formatterClassName);
			Formatter formatter = (Formatter) formatterClass.newInstance();
			return formatter;
		}
		catch (ClassNotFoundException e) {
			throw new FormatterCreationException("Failed to create formatter: " + formatterClassName, e);
		}
		catch (InstantiationException e) {
			throw new FormatterCreationException("Failed to instantiate formatter: " + formatterClassName, e);
		}
		catch (IllegalAccessException e) {
			throw new FormatterCreationException("Bad access writes to formatter: " + formatterClassName, e);
		}

	}
}