package com.chaosserver.bilbo.task.generate;

import com.chaosserver.bilbo.data.DataReaderWriter;
import com.chaosserver.bilbo.data.Directory;
import com.chaosserver.bilbo.data.FilenameMapping;
import com.chaosserver.bilbo.data.Name;
import com.chaosserver.bilbo.data.ReaderWriterException;
import com.chaosserver.bilbo.task.BadTaskCommandArgumentsException;
import com.chaosserver.bilbo.task.BaseTask;
import com.chaosserver.bilbo.task.TaskException;
import com.chaosserver.bilbo.task.generate.joliet.JolietException;
import com.chaosserver.bilbo.task.generate.joliet.JolietFormatter;
import com.chaosserver.bilbo.task.generate.original.OriginalFormatter;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.StringTokenizer;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * Generates mappings files.
 *
 * @author Jordan Reed
 * @version 1.0
 * @since Bilbo 2.0
 */
public class GenerateTask extends BaseTask {
	/** Holds the error response if an error is encountered. */
	protected int errorLevel = 2;

	/** Holds the mapping names to generate for each file. */
	protected Collection mappings;

	/**
	 * Default constructor.
	 */
	public GenerateTask() {
		mappings = new ArrayList();
	}

	/**
	 * Setter for the error level
	 *
	 * @param errorLevel the error level
	 */
	protected void setErrorLevel(int errorLevel) {
		this.errorLevel = errorLevel;
	}

	/**
	 * Getter for the error level
	 *
	 * @return the error level
	 */
	protected int getErrorLevel() {
		return this.errorLevel;
	}

	// Inherits javadocs?
	public void parseTaskArguments(String argumentString) throws BadTaskCommandArgumentsException {
		Pattern p = Pattern.compile("^((?:\\s*\\-\\S+\\s)*)\\s*([^\\-\\s]\\S.*)$");

		Matcher m = p.matcher(argumentString);
		boolean found = m.matches();

		if(!found) {
			throw new BadTaskCommandArgumentsException("Failed to parse argument string: " + argumentString);
		}

		String flags = m.group(1).trim();
		String mappingsString = m.group(2).trim();

		// System.out.println("flags: " + flags);
		// System.out.println("mappings: " + mappingsString);

		parseFlags(flags);
		parseMappingString(mappingsString);

	}

	/**
	 * Parses the flag portion of the task arguments.
	 *
	 * @param flags the flag portion of the task arguments
	 */
	protected void parseFlags(String flags) {
		Pattern p;
		Matcher m;

		p = Pattern.compile("\\-\\S*e(\\d).*");
		m = p.matcher(flags);
		if(m.matches()) {
			int errorLevel = Integer.parseInt(m.group(1));
			setErrorLevel(errorLevel);
		}
	}

	/**
	 * Parses the mapping portion of the task arguments.
	 *
	 * @param mappingsString the mapping portion of the task arguments
	 * @throws BadTaskCommandArgumentsException if there is an error with the arguments provided
	 */
	protected void parseMappingString(String mappingsString) throws BadTaskCommandArgumentsException {
		StringTokenizer stringTokenizer = new StringTokenizer(mappingsString);
		while (stringTokenizer.hasMoreTokens()) {
			String mappingString = stringTokenizer.nextToken();
			// System.out.println("Adding mapping: " + mappingString);
			mappings.add(mappingString);
		}

		if(mappings.size() < 1) {
			throw new BadTaskCommandArgumentsException("There must be at least one mapping");
		}
	}

	/**
	 * Iterators through each file in the directory and generates a mapping
	 * for each mapping passed in through the arguments.
	 *
	 * @param currentDirectory the directory to process
	 * @throws GenerateException if there is an error generating
	 */
	public void execute(File currentDirectory) throws GenerateException {
		File[] files = currentDirectory.listFiles();

		Directory directory = new Directory();
		FormatterFactory formatterFactory = FormatterFactory.getInstance();
		FILE_ITER: for(int i=0; i<files.length; i++) {
			String fileName = files[i].getName();
			if(getSettings().getFileMappingName().equals(fileName)) {
				continue FILE_ITER;
			}

			FilenameMapping filenameMapping = new FilenameMapping();

			Iterator mappingsIterator = mappings.iterator();
			while(mappingsIterator.hasNext()) {
				String mappingName = (String) mappingsIterator.next();
				Formatter formatter = formatterFactory.getFormatter(mappingName);
				Name fullName = new Name(mappingName, formatter.format(directory, files[i]));
				filenameMapping.addName(fullName);
				if(getSettings().isVerbose()) {
					System.out.print(fullName.getStyle());
					System.out.print(" : ");
					System.out.println(fullName.getValue());
				}
			}

			if(getSettings().isVerbose()) {
				System.out.println("");
			}

			// Okay, now add it to the big boy.
			directory.addFilenameMapping(filenameMapping);
		}



		// Marshal and dump
		File mappingFile = new File(currentDirectory, getSettings().getFileMappingName());
		try {
			FileWriter writer = new FileWriter(mappingFile);
			DataReaderWriter dataReaderWriter = new DataReaderWriter();
			dataReaderWriter.write(writer, directory);
			writer.close();
		}
		catch(IOException e) {
			throw new GenerateException("Failed to open file: " + mappingFile.getAbsolutePath());
		}
		catch(ReaderWriterException e) {
			throw new GenerateException("Failed to write to file: " + mappingFile.getAbsolutePath());
		}
	}
}