package com.chaosserver.bilbo.task.generate.joliet;

import com.chaosserver.bilbo.data.Directory;
import com.chaosserver.bilbo.data.FilenameMapping;
import com.chaosserver.bilbo.data.FilenameMappingNotFoundException;
import com.chaosserver.bilbo.task.generate.Formatter;

import java.io.File;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.logging.Logger;

/**
 * Class used to format a given filename into a Joliet acceptable version.
 *
 * <p>
 * A Joliet acceptable version is created by truncating the total filename length,
 * including extension, to be <code>MAX_FILENAME_LENGTH</code> characters long.
 * When a filename is truncated, ~# is placed on the end with the number incrementing
 * from 1 to 9 for each name collision.
 * </p>
 *
 * @author Jordan Reed
 * @version 1.0
 * @since Bilbo 1.0
 */
public class JolietFormatter implements Formatter {
	/** Handle the logging object. */
	protected static Logger logger = Logger.getLogger(JolietFormatter.class.getName());

	/** Max length of the filename in Joliet format. */
	public static final int MAX_FILENAME_LENGTH = 64;

	/**
	 * Generates a valid Joliet filename for the file passed in.
	 *
	 * @param directory The directory the file exists in.  This is needed in case
	 *                  of naming conflicts with files already in the directory.
	 * @param file The file to generate a Joliet name for.
	 *
	 * @return A valid joliet filename for the file
	 */
	public String format(Directory directory, File file) throws JolietException {
		String fullFileName = file.getName();
		String jolietFileName = fullFileName;

		if(fullFileName.length() > MAX_FILENAME_LENGTH) {
			logger.finer("Filename too long: " + fullFileName);
			// We need to chop!  So first let's rip off the extension.

			Pattern p = Pattern.compile("^(.*?)((?:\\.[^\\.]*)?)$");
			Matcher m = p.matcher(fullFileName);
			boolean found = m.matches();

			/*
			System.out.println("Match found?: " + found);
			if(found) {
				for(int i=1; i<= m.groupCount(); i++) {
					System.out.println("Match(" + i + ") = " + m.group(i));
				}
			}
			*/

			if(!found) {
				throw new RuntimeException("Failed to parse the file.  "
					+ "There's a bug in the regular expression I'm "
					+ "using obviously: "
					+ fullFileName);
			}

			String filename  = m.group(1);
			String extension = m.group(2);

			int filenameLength = MAX_FILENAME_LENGTH - 2 - extension.length();
			logger.finest("New filename length: " + filenameLength);

			String shortFileName = filename.substring(0, filenameLength);

			boolean success = false;

			VERSION: for(int version=1; version<10; version++) {
				jolietFileName = shortFileName + "~" + version + extension;
				logger.finer("New filename: " + jolietFileName);

				// Now check if the filename already exists for some other file in the mapping.

				try {
					FilenameMapping result = directory.findFilenameMapping("joliet", jolietFileName);
					continue VERSION;
				}
				catch (FilenameMappingNotFoundException e) {
					// Perfect... we're done.
					success = true;
					break VERSION;
				}

			}

			if(!success) {
				throw new JolietException("Ran out of version to try for file: " + shortFileName + extension);
			}
		}

		return jolietFileName;
	}
}