package com.chaosserver.bilbo.task.validate;

import com.chaosserver.bilbo.data.Directory;
import com.chaosserver.bilbo.data.FilenameMapping;
import com.chaosserver.bilbo.data.FilenameMappingNotFoundException;
import com.chaosserver.bilbo.data.Name;
import com.chaosserver.bilbo.data.NameNotFoundException;
import com.chaosserver.bilbo.data.ReaderWriterException;
import com.chaosserver.bilbo.task.BadTaskCommandArgumentsException;
import com.chaosserver.bilbo.task.BaseTask;

import java.io.File;
import java.io.IOException;
import java.util.Arrays;
import java.util.Collection;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * Looks for errors and inconsistencies in the directory against the mapping file.
 *
 * @author Jordan Reed
 * @version 1.0
 * @since Bilbo 2.0
 */
public class ValidateTask extends BaseTask {
	/** Flag indicating presence of the mapping file in the directory. */
	protected boolean mappingFilePresent = false;

	/** Holds the fromStyle to validate. */
	protected String fromStyle;

	/** Holds the toStyle to validate. */
	protected String toStyle;

	/**
	 * Setter for the fromStyle property.
	 *
	 * @param fromStyle the fromStyle property
	 */
	protected void setFromStyle(String fromStyle) {
		this.fromStyle = fromStyle;
	}

	/**
	 * Getter for the fromStyle property.
	 *
	 * @return fromStyle property
	 */
	protected String getFromStyle() {
		return this.fromStyle;
	}

	/**
	 * Setter for the toStyle property.
	 *
	 * @param toStyle the toStyle property
	 */
	protected void setToStyle(String toStyle) {
		this.toStyle = toStyle;
	}

	/**
	 * Getter for the toStyle property.
	 *
	 * @return the toStyle property
	 */
	protected String getToStyle() {
		return this.toStyle;
	}

	/**
	 * Setter for the mappingFilePresent property.
	 *
	 * @param mappingFilePresent the mappingFilePresent property
	 */
	protected void setMappingFilePresent(boolean mappingFilePresent) {
		this.mappingFilePresent = mappingFilePresent;
	}

	/**
	 * Getter for the mappingFilePresent property.
	 *
	 * @return the mappingFilePresent property
	 */
	public boolean isMappingFilePresent() {
		return this.mappingFilePresent;
	}

	// Inherits javadocs?
	public void parseTaskArguments(String argumentString) throws BadTaskCommandArgumentsException {

		// Split into flags, command, command-flags
		Pattern p = Pattern.compile("^\\s*(\\S+)\\s+(\\S+)\\s*$");
		Matcher m = p.matcher(argumentString);
		boolean found = m.matches();

		if(!found) {
			throw new BadTaskCommandArgumentsException("Failed to parse argument string: " + argumentString);
		}

		String fromStyle = m.group(1);
		String toStyle = m.group(2);

		setFromStyle(fromStyle);
		setToStyle(toStyle);
	}

	/**
	 * Performs a bunch of validation and then spits the results to stdout.
	 *
	 * @param currentDirectory the directory to validate
	 * @throws ValidateException if there is an error validating
	 */
	public void execute(File currentDirectory) throws ValidateException {
		File[] files = currentDirectory.listFiles();

		Collection extraFiles = new LinkedList();
		extraFiles.addAll(Arrays.asList(files));
		Collection extraMappings = new LinkedList();
		Collection valid = new LinkedList();

		File mappingFile = new File(currentDirectory, getSettings().getFileMappingName());
		if(mappingFile.exists()) {
			setMappingFilePresent(true);
			Directory directory = null;

			try {
				directory = Directory.getDirectory(mappingFile);
			}
			catch (ReaderWriterException e) {
				throw new ValidateException("Failed to read in the directory.", e);
			}

			extraMappings.addAll(directory.getFilenameMappings());

			Iterator extraFilesIterator = extraFiles.iterator();
			FILE_ITER: while(extraFilesIterator.hasNext()) {
				File currentFile = (File) extraFilesIterator.next();
				if(currentFile.getName().equals(getSettings().getFileMappingName())) {
					extraFilesIterator.remove();
					continue FILE_ITER;
				}

				try {
					FilenameMapping mapping = directory.findFilenameMapping(fromStyle, currentFile.getName());

					boolean removed = extraMappings.remove(mapping);
					assert (removed) : "Failed to remove from extra";
					extraFilesIterator.remove();
					valid.add(mapping);
				}
				catch (FilenameMappingNotFoundException e) {

				}
			}

		}
		else {
			setMappingFilePresent(false);
		}

		if(!this.isMappingFilePresent()) {
			System.out.println("warning: no mapping file.");
		}

		if(getSettings().isVerbose()) {
			System.out.println("valid: " + valid.size());
		}

		if(extraFiles.size() > 0) {
			System.out.println("warning: files without mapping: " + extraFiles.size());
			if(getSettings().isVerbose()) {
				Iterator extraFilesIterator = extraFiles.iterator();
				while(extraFilesIterator.hasNext()) {
					File currentFile = (File) extraFilesIterator.next();
					System.out.println("  " + currentFile.getName());
				}
			}
		}

		if(extraMappings.size() > 0) {
			System.out.println("warning: mappings without files: " + extraMappings.size());
			if(getSettings().isVerbose()) {
				Iterator extraMappingsIterator = extraMappings.iterator();
				while(extraMappingsIterator.hasNext()) {
					FilenameMapping mapping = (FilenameMapping) extraMappingsIterator.next();

					try {
						Name name = mapping.findName(fromStyle);
						System.out.println("  " + name.getValue());
					}
					catch (NameNotFoundException e) {
						System.out.println("  " + mapping);
					}
				}
			}
		}
	}
}